# Gridlock - a collection of grid-based strategy games
# Copyright (c) 2009 Brian Nenninger (brian@dozingcatsoftware.com)
# Gridlock comes with ABSOLUTELY NO WARRANTY, and is released under version 3 of the GNU GPL.

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *

def set_color_opacity(color, opacity):
     if opacity is not None and opacity!=1:
         # use current color if not provided
         if color is None:
             color = glGetFloatv(GL_CURRENT_COLOR)
         color = list(color[:3]) + [opacity]
     if color is not None:
         glMaterialfv(GL_FRONT,GL_DIFFUSE, color)
         
def do_with_blend(func, blend=False):
    if blend:
        glEnable(GL_BLEND)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
        func()
        glDisable(GL_BLEND)
    else:
        func()
        
def is2d(obj):
    return obj==2 or (hasattr(obj, '__len__') and len(obj)==2)

# these functions are intended to draw pieces in cells, assumes current scale is radius of cell

def draw_sphere(scale=1, color=None, opacity=None, wireframe=False, detail=20):
    func = glutWireSphere if wireframe else glutSolidSphere
    set_color_opacity(color, opacity)
    do_with_blend(lambda: func(scale, detail, detail), opacity is not None)
    
def draw_cube(scale=1, color=None, opacity=None, wireframe=False):
    func = glutWireCube if wireframe else glutSolidCube
    set_color_opacity(color, opacity)
    do_with_blend(lambda: func(scale*1.6), opacity is not None)
    
def draw_torus(scale=1, color=None, opacity=None, wireframe=False, detail=20):
    func = glutWireTorus if wireframe else glutSolidTorus
    set_color_opacity(color, opacity)
    do_with_blend(lambda: func(scale*0.35, scale*0.65, detail, detail), opacity is not None)
    
def draw_diamond(scale=1, color=None, opacity=None, wireframe=False):
    func = glutWireOctahedron if wireframe else glutSolidOctahedron
    glPushMatrix()
    glScalef(scale, scale, scale)
    set_color_opacity(color, opacity)
    do_with_blend(func, opacity is not None)
    glPopMatrix()
    
def draw_cylinder(scale=1, height=0.5, color=None, opacity=None, detail=20, edge=True):
    """ Draws a closed cylinder. If edge is true (the default), the top and bottom caps will have a black edge. """
    quadric = gluNewQuadric()
    set_color_opacity(color, opacity)
    
    def disk():
        set_color_opacity(color, opacity)
        if not edge:
            gluDisk(quadric, 0, scale, detail, detail)
        else:
	        gluDisk(quadric, 0, scale*0.93, detail, detail)
	        set_color_opacity([0,0,0], opacity)
	        gluDisk(quadric, scale*0.93, scale, detail, detail)
    
    def draw():
        gluCylinder(quadric, scale, scale, height*scale, detail, detail)
        glTranslatef(0, 0, height*scale*.01)
        disk()
        glPushMatrix()
        glTranslatef(0, 0, height*scale*.99)
        disk()
        glPopMatrix()
    
    do_with_blend(draw, opacity is not None)
    
def draw_cylinder_or_sphere(dim, scale=1, height=0.5, color=None, opacity=None, detail=20, edge=True):
    if is2d(dim):
        draw_cylinder(scale=scale, height=height, color=color, opacity=opacity, edge=edge, detail=detail)
    else:
        draw_sphere(scale=scale, color=color, opacity=opacity, detail=detail)
    
def draw_teapot(scale=1, color=None, opacity=None, wireframe=False):
    func = glutWireTeapot if wireframe else glutSolidTeapot
    set_color_opacity(color, opacity)
    do_with_blend(lambda: func(scale*0.6), opacity is not None)

def draw_wall(scale=1, flat=True):
    WALL_COLOR = [255,255,255]# if flat else [128, 128, 128] # lighter for black background in 3d, should be in prefs?
    glPushMatrix()
    glDisable(GL_DITHER)
    glDisable(GL_LIGHTING)
    glColor3ub(*WALL_COLOR)
    if flat:
        glTranslatef(0, 0, 0.3*scale*1.01)
        glScalef(1, 1, 0.3)
    glutSolidCube(scale*2)
    # attempting to outline edges, but looks terrible
    #glColor3ub(0,0,0)
    #glutWireCube(scale*2)
    
    glEnable(GL_DITHER)
    glEnable(GL_LIGHTING)
    glPopMatrix()
    
def draw_blobs(dim, blobs, scale=1, height=0.5, color=None, opacity=None, detail=20, edge=False):
    """ Draws a sequence of blobs, where blobs is an iterable of ((x,y[,z]), size) """
    set_color_opacity(color, opacity)
    for pos,size in blobs:
        glPushMatrix()
        glTranslatef(scale*pos[0], scale*pos[1], scale*pos[2] if len(pos)>2 else 0)
        draw_cylinder_or_sphere(dim, scale=size*scale, height=height, opacity=opacity, detail=detail, edge=edge)
        glPopMatrix()

def draw_cube_overlay(dim, color, scale=1, opacity=0.2):        
    glPushMatrix()
    set_color_opacity(color, opacity)
    # in 2d, draw short "cube" just big enough to cover pieces, in 3d shrink to not obscure selection cube
    if is2d(dim):
        glTranslatef(0, 0, 0.205*scale) # slightly above board to avoid flickering
        glScalef(1, 1, 0.2)
    else:
        scale *= 0.75
    do_with_blend(lambda: glutSolidCube(scale*2), blend=True)
    glPopMatrix()
